@@ -23,6 +23,8 @@ extern "C" {
 
 #include <CL/cl.h>
 #include <CL/cl_gl.h>
+#include <CL/cl_exp_tensor.h>
+#include <CL/cl_exp_defined_builtin_kernels.h>
 #include <CL/cl_ext.h>
 #include <CL/cl_ext_pocl.h>
 
