@@ -25,8 +25,6 @@
 #include "pocl_cl.h"
 #include "pocl_util.h"
 
-extern unsigned long sampler_c;
-
 CL_API_ENTRY cl_sampler CL_API_CALL
 POname(clCreateSampler)(cl_context          context,
                 cl_bool             normalized_coords, 
@@ -61,7 +59,7 @@ CL_API_SUFFIX__VERSION_1_0
   sampler = (cl_sampler)calloc (1, sizeof (struct _cl_sampler));
   POCL_GOTO_ERROR_COND ((sampler == NULL), CL_OUT_OF_HOST_MEMORY);
 
-  POCL_INIT_OBJECT (sampler);
+  POCL_INIT_OBJECT (sampler, context);
   POname (clRetainContext) (context);
   sampler->context = context;
   sampler->normalized_coords = normalized_coords;
