@@ -1,3 +1,26 @@
+/* OpenCL runtime library: clEnqueueCopyBufferToImage()
+
+   Copyright (c) 2011-2024 PoCL developers
+
+   Permission is hereby granted, free of charge, to any person obtaining a copy
+   of this software and associated documentation files (the "Software"), to
+   deal in the Software without restriction, including without limitation the
+   rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+   sell copies of the Software, and to permit persons to whom the Software is
+   furnished to do so, subject to the following conditions:
+
+   The above copyright notice and this permission notice shall be included in
+   all copies or substantial portions of the Software.
+
+   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+   IN THE SOFTWARE.
+*/
+
 #include "pocl_image_util.h"
 #include "pocl_mem_management.h"
 #include "pocl_shared.h"
@@ -17,7 +40,6 @@ pocl_copy_buffer_to_image_common (
     cl_event *event,
     const cl_sync_point_khr *sync_point_wait_list,
     cl_sync_point_khr *sync_point,
-    cl_mutable_command_khr *mutable_handle,
     _cl_command_node **cmd)
 {
   cl_int errcode;
@@ -50,10 +72,10 @@ pocl_copy_buffer_to_image_common (
       else
         {
           return POname (clCommandCopyBufferRectKHR) (
-              command_buffer, command_queue, src_buffer, dst_image->buffer,
-              src_origin, i1d_origin, i1d_region, dst_image->image_row_pitch,
-              0, dst_image->image_row_pitch, 0, num_items_in_wait_list,
-              sync_point_wait_list, sync_point, mutable_handle);
+            command_buffer, command_queue, NULL, src_buffer, dst_image->buffer,
+            src_origin, i1d_origin, i1d_region, dst_image->image_row_pitch, 0,
+            dst_image->image_row_pitch, 0, num_items_in_wait_list,
+            sync_point_wait_list, sync_point, cmd);
         }
     }
 
@@ -69,20 +91,14 @@ pocl_copy_buffer_to_image_common (
   if (errcode != CL_SUCCESS)
     return errcode;
 
-  POCL_CONVERT_SUBBUFFER_OFFSET (src_buffer, src_offset);
-
   POCL_GOTO_ERROR_ON (
       (src_buffer->size > command_queue->device->max_mem_alloc_size),
       CL_OUT_OF_RESOURCES, "src is larger than device's MAX_MEM_ALLOC_SIZE\n");
 
   _cl_command_node *c = *cmd;
   cl_device_id dev = command_queue->device;
-  c->command.write_image.dst_mem_id
-      = &dst_image->device_ptrs[dev->global_mem_id];
   c->command.write_image.dst = dst_image;
   c->command.write_image.src_host_ptr = ((void *)0);
-  c->command.write_image.src_mem_id
-      = &src_buffer->device_ptrs[dev->global_mem_id];
   c->command.write_image.src = src_buffer;
   c->command.write_image.src_row_pitch = src_row_pitch;
   c->command.write_image.src_slice_pitch = src_slice_pitch;
@@ -123,9 +139,8 @@ CL_API_SUFFIX__VERSION_1_0
                           CL_DEVICE_NOT_AVAILABLE);
 
   errcode = pocl_copy_buffer_to_image_common (
-      NULL, command_queue, src_buffer, dst_image, src_offset, dst_origin,
-      region, num_events_in_wait_list, event_wait_list, event, NULL, NULL,
-      NULL, &cmd);
+    NULL, command_queue, src_buffer, dst_image, src_offset, dst_origin, region,
+    num_events_in_wait_list, event_wait_list, event, NULL, NULL, &cmd);
 
   if (errcode != CL_SUCCESS)
     return errcode;
