@@ -47,9 +47,9 @@ CL_API_SUFFIX__VERSION_1_2
   if (errcode != CL_SUCCESS)
     return errcode;
 
-  errcode = pocl_create_command (&cmd, command_queue, CL_COMMAND_MARKER, event,
-                                 num_events_in_wait_list, event_wait_list, 0,
-                                 NULL, NULL);
+  errcode
+    = pocl_create_command (&cmd, command_queue, CL_COMMAND_MARKER, event,
+                           num_events_in_wait_list, event_wait_list, NULL);
   if (errcode != CL_SUCCESS)
     goto ERROR;
 
