@@ -71,9 +71,9 @@ POname(clEnqueueSVMFree) (cl_command_queue command_queue,
 
   _cl_command_node *cmd = NULL;
 
-  errcode = pocl_create_command (&cmd, command_queue, CL_COMMAND_SVM_FREE,
-                                 event, num_events_in_wait_list,
-                                 event_wait_list, 0, NULL, NULL);
+  errcode
+    = pocl_create_command (&cmd, command_queue, CL_COMMAND_SVM_FREE, event,
+                           num_events_in_wait_list, event_wait_list, NULL);
 
   if (errcode != CL_SUCCESS)
     {
