@@ -27,7 +27,8 @@ CL_API_ENTRY cl_int CL_API_CALL POname (clGetEventInfo) (
     cl_event event, cl_event_info param_name, size_t param_value_size,
     void *param_value, size_t *param_value_size_ret) CL_API_SUFFIX__VERSION_1_0
 {
-  POCL_RETURN_ERROR_COND((event == NULL), CL_INVALID_EVENT);
+  POCL_RETURN_ERROR_COND ((!IS_CL_OBJECT_VALID (event)), CL_INVALID_EVENT);
+
   POCL_LOCK_OBJ (event);
   cl_int s = event->status;
   cl_command_queue q = event->queue;
