@@ -246,8 +246,6 @@ POname(clGetProgramInfo)(cl_program program,
           /* only when param_value is non-NULL */
           if (size > param_value_size)
             return CL_INVALID_VALUE;
-          /* should not break from the switch clause
-             because of POCL_ABORT_UNIMPLEMENTED */
           for (i = 0; i < num_kernels; ++i)
             {
               if (i == 0)
@@ -263,8 +261,8 @@ POname(clGetProgramInfo)(cl_program program,
       return CL_SUCCESS;
     }
   default:
-    POCL_RETURN_ERROR_ON(1, CL_INVALID_VALUE,
-                         "Parameter %i not implemented\n", param_name);
+    POCL_RETURN_ERROR (CL_INVALID_VALUE, "Parameter %i not implemented\n",
+                       param_name);
   }
 
 }
