@@ -37,7 +37,7 @@ typedef struct pool_thread_data thread_data;
 /* Initializes scheduler. Must be called before any kernel enqueue */
 cl_int pthread_scheduler_init (cl_device_id device);
 
-void pthread_scheduler_uninit (cl_device_id device);
+void pthread_scheduler_uninit ();
 
 /* Gives ready-to-execute command for scheduler */
 void pthread_scheduler_push_command (_cl_command_node *cmd);
