@@ -1,12 +1,12 @@
 /* pocl_debug.h: Internal debugging aids.
 
-   Copyright (c) 2011-2021 pocl developers
+   Copyright (c) 2011-2024 pocl developers
 
    Permission is hereby granted, free of charge, to any person obtaining a copy
-   of this software and associated documentation files (the "Software"), to deal
-   in the Software without restriction, including without limitation the rights
-   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-   copies of the Software, and to permit persons to whom the Software is
+   of this software and associated documentation files (the "Software"), to
+   deal in the Software without restriction, including without limitation the
+   rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+   sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:
 
    The above copyright notice and this permission notice shall be included in
@@ -16,40 +16,43 @@
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-   THE SOFTWARE.
+   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+   IN THE SOFTWARE.
 */
 
 #ifndef POCL_DEBUG_H
 #define POCL_DEBUG_H
 
+#include <stddef.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#ifdef _WIN32
-#  include <stdint.h>
-#  include <stddef.h> // size_t
-#  define PRIu64 "I64u"
-#  define PRIX64 "I64x"
-#  define PRIXPTR "p"
-#  define PRIuS "Iu"
-#else
-# ifndef __STDC_FORMAT_MACROS
-# define __STDC_FORMAT_MACROS
-# endif
-# include <inttypes.h>
+#ifndef __STDC_FORMAT_MACROS
+#define __STDC_FORMAT_MACROS
 #endif
+#include <inttypes.h>
 
 #include "config.h"
 
+#include "CL/cl.h"
 #include "pocl_export.h"
+#include "pocl_threads.h"
 
 // size_t print spec
 #ifndef PRIuS
+#ifdef _MSC_VER
+#define PRIuS "Iu"
+#else
 # define PRIuS "zu"
 #endif
+#endif
+
+#ifndef PRId64
+# define PRId64 PRIu64
+#endif
 
 #if defined(__ANDROID__)
 #include <android/log.h>
@@ -167,6 +170,31 @@ POCL_EXPORT
 
         #define POCL_DEBUG_HEADER(FILTER, FILTER_TYPE) \
             pocl_debug_print_header (__func__, __LINE__, #FILTER, FILTER_TYPE);
+/**
+ * Dumps all commands and events of the context as a graphviz dot file.
+ *
+ * Currently works only with PoCL contexts.
+ *
+ * \param context the OpenCL context to dump.
+ * \param file_name the target file name.
+ */
+POCL_EXPORT
+void pocl_dump_dot_task_graph (cl_context context, const char *file_name);
+
+POCL_EXPORT
+void pocl_dump_dot_task_graph_wait ();
+
+POCL_EXPORT
+void pocl_dump_dot_task_graph_signal ();
+
+/**
+ * Converts a command type to a string.
+ *
+ * \param cmd The command.
+ * \param shortened Set to 1 for a shortened string.
+ */
+POCL_EXPORT
+const char *pocl_command_type_to_str (cl_command_type cmd, int shortened);
 
 POCL_EXPORT
         extern void pocl_debug_output_lock (void);
@@ -467,6 +495,14 @@ POCL_EXPORT
 
 #endif
 
+#define POCL_RETURN_ERROR(err_code, ...)                                      \
+        POCL_MSG_ERR2 (#err_code, __VA_ARGS__);                               \
+        return err_code
+
+#define POCL_GOTO_ERROR(err_code, ...)                                        \
+        POCL_MSG_ERR2 (#err_code, __VA_ARGS__);                               \
+        errcode = err_code;                                                   \
+        goto err_code
 
 #define POCL_GOTO_ERROR_ON(cond, err_code, ...)                             \
   do                                                                        \
@@ -480,16 +516,16 @@ POCL_EXPORT
     }                                                                       \
   while (0)
 
-#define POCL_RETURN_ERROR_ON(cond, err_code, ...)                           \
-  do                                                                        \
-    {                                                                       \
-      if (cond)                                                             \
-        {                                                                   \
-            POCL_MSG_ERR2(#err_code, __VA_ARGS__);                          \
-            return err_code;                                                \
-        }                                                                   \
-    }                                                                       \
-  while (0)
+#define POCL_RETURN_ERROR_ON(cond, err_code, ...)                             \
+        do                                                                    \
+          {                                                                   \
+            if (cond)                                                         \
+              {                                                               \
+                POCL_MSG_ERR2 (#err_code, __VA_ARGS__);                       \
+                return err_code;                                              \
+              }                                                               \
+          }                                                                   \
+        while (0)
 
 #define POCL_RETURN_ERROR_COND(cond, err_code)                              \
   do                                                                        \
@@ -538,8 +574,10 @@ POCL_EXPORT
     }                                                                         \
   while (0)
 
+
 #ifdef __cplusplus
 }
 #endif
 
+
 #endif
