@@ -1,4 +1,3 @@
-
 #!/usr/bin/python3
 #
 # A script to generate wrapping functions (SPIR-mangled with SPIR AS) that will wrap
@@ -39,7 +38,7 @@ parser = argparse.ArgumentParser(description='SPIR wrapper generator.')
 
 parser.add_argument('-t', '--target',
 	dest='target',
-	default="cpu_x86", choices=["cpu_x86", "cpu_arm", "cuda"],
+	default="cpu_x86", choices=["cpu_x86", "cpu_arm", "cuda", "cpu_riscv"],
 	help='target to generate wrapper for')
 
 parser.add_argument('-r', '--register-size',
@@ -51,11 +50,6 @@ parser.add_argument('output',
 	type=argparse.FileType('w', encoding='utf-8'),
 	help='output file')
 
-parser.add_argument('-q', '--opaque-pointers',
-	dest='opaque_ptrs',
-	action='store_true',
-	help="enable LLVM's opaque pointers")
-
 parser.add_argument('-g', '--generic-as',
 	dest='generic_as',
 	action='store_true',
@@ -83,13 +77,17 @@ AS_CASTS_REQUIRED = args.target.startswith('cpu')
 # use of byval/sret for args that don't fit into registers
 X86_CALLING_ABI = (args.target == 'cpu_x86')
 ARM_CALLING_ABI = (args.target == 'cpu_arm')
+RISCV64_CALLING_ABI = (args.target == 'cpu_riscv')
 SPIR_CALLING_ABI = (args.target == 'cuda')
 
 # size of the largest CPU (SIMD) register. Values larger than this
 # will be passed with byval/sret
 CPU_ABI_REG_SIZE = args.reg_size
+# for RISC-V the ABI is always 128bit max size
+if RISCV64_CALLING_ABI:
+	CPU_ABI_REG_SIZE = 128
 
-OPAQUE_POINTERS = args.opaque_ptrs
+OPAQUE_POINTERS = True
 
 GENERIC_AS = args.generic_as
 
@@ -334,6 +332,51 @@ elif ARM_CALLING_ABI:
 
 		"<3 x i32>": "<4 x i32>",
 	}
+elif RISCV64_CALLING_ABI:
+	COERCE_VECTOR_MAP = {
+		"<2 x i8>": "i64",
+		"<3 x i8>": "i64",
+		"<4 x i8>": "i64",
+		"<8 x i8>": "i64",
+		"<16 x i8>": "i128",
+
+		"<2 x i16>": "i64",
+		"<3 x i16>": "i64",
+		"<4 x i16>": "i64",
+		"<8 x i16>": "i128",
+
+		"<2 x half>": "i64",
+		"<3 x half>": "i64",
+		"<4 x half>": "i64",
+		"<8 x half>": "i128",
+
+		"<2 x i32>": "i64",
+		"<3 x i32>": "i128",
+		"<4 x i32>": "i128",
+
+		"<2 x float>": "i64",
+		"<3 x float>": "i128",
+		"<4 x float>": "i128",
+
+		"<2 x i64>": "i128",
+
+		"<2 x double>": "i128",
+	}
+	SHUFFLE_VECTOR_MAP = {
+		"<2 x i8>": "<8 x i8>",
+		"<3 x i8>": "<8 x i8>",
+		"<4 x i8>": "<8 x i8>",
+
+		"<2 x i16>": "<4 x i16>",
+		"<3 x i16>": "<4 x i16>",
+
+		"<2 x half>": "<4 x half>",
+		"<3 x half>": "<4 x half>",
+
+		"<3 x i32>": "<4 x i32>",
+
+		"<3 x float>": "<4 x float>",
+        }
 else:
 	COERCE_VECTOR_MAP = {
 	}
@@ -575,21 +618,57 @@ ALREADY_DECLARED = {
 }
 
 # some coerced args require a shuffle b/c different size in bits
-def shuffle_coerced_arg(llvm_i, input_type, input_arg, all_instr):
-	if input_type.startswith("<3"):
+# if the argument needs shuffling, inserts the shufle to `all_instrs` and return the new (shuffled) value;
+# otherwise return input_arg
+def shuffle_coerced_arg(llvm_i, input_type, coerced_arg_type, input_arg, all_instr):
+	if (not RISCV64_CALLING_ABI) and input_type.startswith("<3"):
 		four_vec = "<4" + input_type[2:]
-		all_instr.append("  %%%u = shufflevector %s, %s undef, <4 x i32> <i32 0, i32 1, i32 2, i32 undef>" % (llvm_i, input_arg, input_type))
+		all_instr.append("  %%%u = shufflevector %s, %s poison, <4 x i32> <i32 0, i32 1, i32 2, i32 poison>" % (llvm_i, input_arg, input_type))
 		shuffled_arg = four_vec + " %" + chr(48+llvm_i)
 		llvm_i += 1
+        # for ARM, shuffle 2xi8 -> 4xi8
 	elif ARM_CALLING_ABI and input_type == "<2 x i8>":
 		four_vec = "<4" + input_type[2:]
-		all_instr.append("  %%%u = shufflevector %s, %s undef, <4 x i32> <i32 0, i32 1, i32 undef, i32 undef>" % (llvm_i, input_arg, input_type))
+		all_instr.append("  %%%u = shufflevector %s, %s poison, <4 x i32> <i32 0, i32 1, i32 poison, i32 poison>" % (llvm_i, input_arg, input_type))
 		shuffled_arg = four_vec + " %" + chr(48+llvm_i)
 		llvm_i += 1
+	elif RISCV64_CALLING_ABI and (input_type in SHUFFLE_VECTOR_MAP):
+		new_vec = SHUFFLE_VECTOR_MAP[input_type]
+		num_elem = ord(new_vec[1]) - 48
+		num_input = ord(input_type[1]) - 48
+		num_undef = num_elem - num_input
+		def_elems = ", ".join(["i32 "+str(i) for i in range(0,num_input)])
+		undef_elems = ", ".join(["i32 poison" for i in range(0,num_undef)])
+		all_instr.append("  %%%u = shufflevector %s, %s poison, <%u x i32> <%s, %s>" % (llvm_i, input_arg, input_type, num_elem, def_elems, undef_elems))
+		shuffled_arg = new_vec + " %" + chr(48+llvm_i)
+		llvm_i += 1
 	else:
 		shuffled_arg = input_arg
 	return (shuffled_arg, llvm_i)
 
+def unshuffle_coerced_ret_type(coerced_ret_type, ret_type, coerced_name, retval_name):
+	if (not RISCV64_CALLING_ABI) and ret_type.startswith("<3"):
+		four_vec = "<4" + ret_type[2:]
+                # %bc_ret = bitcast i64 %%coerced_ret to <4 x i32>"
+		bitcast_inst = "  %%bc_ret = bitcast %s %%%s to %s \n" % (coerced_ret_type, coerced_name, four_vec)
+                # %final_ret = shufflevector <4 x i32> %bc_ret,  <4 x i32> poison, <0, 1, 2>
+		shuffle_inst = "  %%%s = shufflevector %s %%bc_ret, %s poison, <3 x i32> <i32 0, i32 1, i32 2>\n" % (retval_name, four_vec, four_vec)
+		return bitcast_inst + shuffle_inst
+
+	elif RISCV64_CALLING_ABI and (ret_type in SHUFFLE_VECTOR_MAP):
+		# e.g. ret_type = 2 x i8, coerced_ret_type = i64, bitcast_type = 8 x i8
+		bitcast_type = SHUFFLE_VECTOR_MAP[ret_type]
+                # %bc_ret = bitcast i64 %%coerced_ret to <8 x i8>"
+		bitcast_inst = "  %%bc_ret = bitcast %s %%%s to %s \n" % (coerced_ret_type, coerced_name, bitcast_type)
+		num_input = ord(ret_type[1]) - 48
+		def_elems = ", ".join(["i32 "+str(i) for i in range(0,num_input)])
+                # <2 x i8> %final_ret = shufflevector <8 x i8> %bc_ret,  <8 x i8> poison, <0, 1>
+		shuffle_inst = "  %%%s = shufflevector %s %%bc_ret, %s poison, <%u x i32> <%s>\n" % (retval_name, bitcast_type, bitcast_type, num_input, def_elems)
+		return bitcast_inst + shuffle_inst
+
+	else:
+		return None
+
 # returns a LLVM type with addrspace
 # e.g. for argtype=="Pi" returns "i32 *"
 def llvm_arg_type(argtype, AS):
@@ -653,7 +732,8 @@ def replace_arg_type(argtype, replacement):
 
 # coerce vector type arguments (leave other types alone)
 def coerce_llvm_vector_type(type):
-	if (not X86_CALLING_ABI) and (not ARM_CALLING_ABI):
+	# only for ARM & X86 & RISCV, not SPIR
+	if SPIR_CALLING_ABI:
 		return type
 	if type in COERCE_VECTOR_MAP:
 		return COERCE_VECTOR_MAP[type]
@@ -662,18 +742,21 @@ def coerce_llvm_vector_type(type):
 
 # get arg type for types larger than cpu reg size
 def byval_llvm_vector_type(type):
-	if (not X86_CALLING_ABI) and (not ARM_CALLING_ABI):
+	# only for ARM & X86 & RISCV, not SPIR
+	if SPIR_CALLING_ABI:
 		return type
 	if type not in BYVAL_VECTOR_MAP:
 		return type
 	if X86_CALLING_ABI:
 		return BYVAL_VECTOR_MAP[type]
-	if ARM_CALLING_ABI:
+	if ARM_CALLING_ABI or RISCV64_CALLING_ABI:
 		return type+"*"
+	return None
 
-# return type, only required for ARM cpu
+# return type, only required for ARM & RISCV cpu
 def sret_llvm_vector_type(type):
-	if not ARM_CALLING_ABI:
+	# only for RISCV & ARM, not X86 & SPIR
+	if (not RISCV64_CALLING_ABI) and (not ARM_CALLING_ABI):
 		return type
 	if type not in SRET_VECTOR_MAP:
 		return type
@@ -729,21 +812,21 @@ def generate_function(name, ret_type, ret_type_ext, multiAS, *args):
 		# instr index, LLVM IR expects instructions to be numbered from 1
 		llvm_i = 1
 
-		# arg without qualifiers, saved for mangling name compression
-		last_pure_arg = None
+		# args without qualifiers, saved for mangling name compression
+		saved_pure_args = []
 
-		# ARM does not coerce return types, x86-64 does
+		# ARM does not coerce return types, x86-64 & RISC-V does
 		if ARM_CALLING_ABI:
 			coerced_ret_type = ret_type
 		else:
 			coerced_ret_type = coerce_llvm_vector_type(ret_type)
 
-		# handle sret returvn value for ARM
+		# handle sret returvn value for ARM & RISCV
 		# if the function uses even one sret type argument, it must also return with sret
 		sret_ret_type = sret_llvm_vector_type(ret_type)
 		retval_alloca_inst = None
 		retval_align = None
-		if ARM_CALLING_ABI and (ret_type != sret_ret_type):
+		if sret_ret_type and (ret_type != sret_ret_type):
 			# add alloca for retval
 			alignment = sret_ret_type.find("align")
 			if alignment > 0:
@@ -757,6 +840,10 @@ def generate_function(name, ret_type, ret_type_ext, multiAS, *args):
 			llvm_i += 1
 
 		####### process args
+		if not args:
+			spir_mangled_func_suffix.append("v")
+			ocl_mangled_func_suffix.append("v")
+			args = []
 		for cast in args:
 
 			if arg_addr_spaces:
@@ -768,11 +855,25 @@ def generate_function(name, ret_type, ret_type_ext, multiAS, *args):
 			#   Dv2_cPULocalDv2_c -> Dv2_cPULocalS_
 			pure_arg = pure_arg_type(cast)
 			actual_arg = cast
-			if pure_arg == last_pure_arg and last_pure_arg.startswith("D") and (not last_pure_arg.startswith("Dh")):
-				actual_arg = replace_arg_type(actual_arg, "S_")
-			if pure_arg == last_pure_arg and last_pure_arg == '12memory_order':
-				actual_arg = replace_arg_type(actual_arg, "S4_")
-			last_pure_arg = pure_arg
+
+			replaced = False
+			if len(pure_arg) > 1 and pure_arg != "Dh":
+				for idx, prev_arg in enumerate(saved_pure_args):
+					if pure_arg != prev_arg:
+						continue
+					compressed_arg = "S_"
+					if idx > 0:
+						# _Z16_cl_write_imagei14ocl_image3d_woDv4_i -> S0_
+						# _Z41_cl_atomic_compare_exchange_weak_explicitPU8CLglobalVU7_AtomicjPU9CLprivatejj12memory_order -> S4
+						if actual_arg == "12memory_order":
+							compressed_arg = "S" + str(idx+1) + "_"
+						else:
+							compressed_arg = "S" + str(idx-1) + "_"
+					actual_arg = replace_arg_type(actual_arg, compressed_arg)
+					replaced = True
+					break
+			if not replaced:
+				saved_pure_args.append(pure_arg)
 
 			# convert arg type to SPIR mangled type with AS
 			# e.g. "Pi" -> "PU3AS3i"
@@ -790,7 +891,7 @@ def generate_function(name, ret_type, ret_type_ext, multiAS, *args):
 			####### generate caller (SPIR wrapper function) llvm arg types
 			# get LLVM type name (e.g. <2 x i32>) for arg
 			spir_arg_type = llvm_arg_type(cast, LLVM_SPIR_AS[AS])
-			# get coerced type
+			# get coerced type e.g. i64
 			coerced_arg_type = coerce_llvm_vector_type(spir_arg_type)
 			# get byval type
 			byval_sret_arg_type = byval_llvm_vector_type(spir_arg_type)
@@ -809,7 +910,7 @@ def generate_function(name, ret_type, ret_type_ext, multiAS, *args):
 			# this shouldn't interact with addrspace casts.
 			if coerced_arg_type != spir_arg_type:
 				# some vectors require a shuffle
-				shuffled_arg, llvm_i = shuffle_coerced_arg(llvm_i, spir_arg_type, noext_caller_arg, all_instr)
+				shuffled_arg, llvm_i = shuffle_coerced_arg(llvm_i, spir_arg_type, coerced_arg_type, noext_caller_arg, all_instr)
 				all_instr.append("  %%%u = bitcast %s to %s" % (llvm_i, shuffled_arg, coerced_arg_type))
 				callee_args.append(coerced_arg_type + " %" + chr(48+llvm_i))
 				decl_args.append(coerced_arg_type)
@@ -842,7 +943,6 @@ def generate_function(name, ret_type, ret_type_ext, multiAS, *args):
 				decl_args.append(ocl_arg_type)
 			arg_i += 1
 
-
 		######## generate final mangled function names
 		spir_mangled_func_suffix = "".join(spir_mangled_func_suffix)
 		ocl_mangled_func_suffix = "".join(ocl_mangled_func_suffix)
@@ -859,7 +959,7 @@ def generate_function(name, ret_type, ret_type_ext, multiAS, *args):
 			decl_ret_type = 'void'
 		else:
 			decl_ret_type = coerced_ret_type
-		
+
 		#if GENERIC_AS and GEN_AS_CALLEE_IDENTICAL and (AS == "generic") and ("private" in addr_spaces):
 		declaration = "declare %s %s(%s) local_unnamed_addr #0" % (decl_ret_type, ocl_mangled_name, decl_args)
 		if declaration in ALREADY_DECLARED.keys():
@@ -880,10 +980,9 @@ def generate_function(name, ret_type, ret_type_ext, multiAS, *args):
 		else:
 			if ret_type != coerced_ret_type:
 				print("  %%coerced_ret = call %s %s(%s)" % (coerced_ret_type, ocl_mangled_name, callee_args))
-				if ret_type.startswith("<3"):
-					four_vec = "<4" + ret_type[2:]
-					print("  %%bc_ret = bitcast %s %%coerced_ret to %s" % (coerced_ret_type, four_vec))
-					print("  %%final_ret = shufflevector %s %%bc_ret, %s undef, <3 x i32> <i32 0, i32 1, i32 2>" % (four_vec, four_vec))
+				unshuffle_insts = unshuffle_coerced_ret_type(coerced_ret_type, ret_type, "coerced_ret", "final_ret")
+				if unshuffle_insts:
+					print(unshuffle_insts)
 				else:
 					print("  %%final_ret = bitcast %s %%coerced_ret to %s" % (coerced_ret_type, ret_type))
 
@@ -1277,8 +1376,8 @@ for arg_type in ['c', 'h', 's', 't', 'i', 'j', 'l', 'm', 'f', 'd']:
 	else:
 		uarg_type = arg_type
 
-	for vector_size in [2,3,4,8,16]:
-		for perm_size in [2,3,4,8,16]:
+	for vector_size in [2,4,8,16]:
+		for perm_size in [2,4,8,16]:
 			ret_type = "Dv" + str(perm_size) + "_" + arg_type
 			in_type = "Dv" + str(vector_size) + "_" + arg_type
 			mask_type = "Dv" + str(perm_size) + "_" + uarg_type
@@ -1286,10 +1385,9 @@ for arg_type in ['c', 'h', 's', 't', 'i', 'j', 'l', 'm', 'f', 'd']:
 			generate_function("shuffle2", SIG_TO_LLVM_TYPE_MAP[ret_type], '', False, in_type, in_type, mask_type)
 
 # convert
+CONVERT_TYPES = ['c', 'h', 's', 't', 'i', 'j', 'l', 'm', 'f', 'd']
 if FP16:
-	CONVERT_TYPES = ['c', 'h', 's', 't', 'i', 'j', 'l', 'm', 'f', 'd', 'Dh']
-else:
-	CONVERT_TYPES = ['c', 'h', 's', 't', 'i', 'j', 'l', 'm', 'f', 'd']
+	CONVERT_TYPES.append('Dh')
 
 for dst_type in CONVERT_TYPES:
 	for src_type in CONVERT_TYPES:
@@ -1308,10 +1406,16 @@ for dst_type in CONVERT_TYPES:
 						signext = ''
 					generate_function('convert_'+SIG_TO_TYPE_NAME_MAP[dst_t]+sat+rounding, SIG_TO_LLVM_TYPE_MAP[dst_t], signext, False, src_t)
 
+if FP16:
+	COLOR_TYPES = ['i', 'ui', 'f', 'h']
+else:
+	COLOR_TYPES = ['i', 'ui', 'f']
+
 IMG_COLOR_TYPE_MAP = {
 	'i' : 'Dv4_i',
 	'ui' : 'Dv4_j',
 	'f' : 'Dv4_f',
+	'h' : 'Dv4_Dh'
 }
 
 IMG_COORD_SIZE = {
@@ -1336,7 +1440,7 @@ for img_type in ['image1d', 'image2d', 'image3d', 'image1d_array', 'image2d_arra
 		img_type_llvm = 'ocl_' + img_type + "_" + access
 		img_type_llvm = str(len(img_type_llvm)) + img_type_llvm
 
-		for color_type in ['i', 'ui', 'f']:
+		for color_type in COLOR_TYPES:
 			# return value for read_image or input value for write_image
 			mang_color_type = IMG_COLOR_TYPE_MAP[color_type]
 
@@ -1480,6 +1584,77 @@ generate_function("atomic_work_item_fence", SIG_TO_LLVM_TYPE_MAP['v'], '', None,
 generate_function("work_group_barrier", SIG_TO_LLVM_TYPE_MAP['v'], '', None, 'j', '12memory_scope')
 generate_function("work_group_barrier", SIG_TO_LLVM_TYPE_MAP['v'], '', None, 'j')
 
+# generate wrapper function
+#def generate_function(name, ret_type, ret_type_ext, multiAS, *args):
+# subgroups
+generate_function("get_sub_group_size", SIG_TO_LLVM_TYPE_MAP['i'], '', None)
+generate_function("get_max_sub_group_size", SIG_TO_LLVM_TYPE_MAP['i'], '', None)
+generate_function("get_num_sub_groups", SIG_TO_LLVM_TYPE_MAP['i'], '', None)
+generate_function("get_enqueued_num_sub_groups", SIG_TO_LLVM_TYPE_MAP['i'], '', None)
+generate_function("get_sub_group_id", SIG_TO_LLVM_TYPE_MAP['i'], '', None)
+generate_function("get_sub_group_local_id", SIG_TO_LLVM_TYPE_MAP['i'], '', None)
+generate_function("sub_group_ballot", SIG_TO_LLVM_TYPE_MAP['Dv4_i'], '', None, 'i')
+generate_function("sub_group_any", SIG_TO_LLVM_TYPE_MAP['i'], '', None, 'i')
+generate_function("sub_group_all", SIG_TO_LLVM_TYPE_MAP['i'], '', None, 'i')
+generate_function("sub_group_barrier", SIG_TO_LLVM_TYPE_MAP['v'], '', None, 'j')
+generate_function("sub_group_barrier", SIG_TO_LLVM_TYPE_MAP['v'], '', None, 'j', '12memory_scope')
+
+SUBGROUP_TYPES = ['c', 'h', 's', 't', 'i', 'j', 'l', 'm', 'f', 'd']
+if FP16:
+	SUBGROUP_TYPES.append('Dh')
+
+for arg_type in SUBGROUP_TYPES:
+	ret_type = arg_type
+	signext = LLVM_TYPE_EXT_MAP[ret_type]
+	mask_type = 'j'
+	for suffix in ['', '_xor']:
+		generate_function("sub_group_shuffle"+suffix, SIG_TO_LLVM_TYPE_MAP[ret_type], signext, False, arg_type, mask_type)
+		generate_function("intel_sub_group_shuffle"+suffix, SIG_TO_LLVM_TYPE_MAP[ret_type], signext, False, arg_type, mask_type)
+	generate_function("sub_group_broadcast", SIG_TO_LLVM_TYPE_MAP[ret_type], signext, False, arg_type, mask_type)
+	for suffix in ['_add', '_min', '_max']:
+		generate_function("sub_group_reduce"+suffix, SIG_TO_LLVM_TYPE_MAP[ret_type], signext, False, arg_type)
+		generate_function("sub_group_scan_inclusive"+suffix, SIG_TO_LLVM_TYPE_MAP[ret_type], signext, False, arg_type)
+		generate_function("sub_group_scan_exclusive"+suffix, SIG_TO_LLVM_TYPE_MAP[ret_type], signext, False, arg_type)
+	for suffix in ['_up', '_down']:
+		generate_function("intel_sub_group_shuffle"+suffix, SIG_TO_LLVM_TYPE_MAP[ret_type], signext, False, arg_type, arg_type, mask_type)
+
+# Intel extension adds support for some types which are not supported by the Khronos extension:
+# For the sub_group_shuffle, sub_group_shuffle_down, sub_group_shuffle_up, and sub_group_shuffle_xor functions, gentype is float, float2, float3,
+# float4, float8, float16, int, int2, int3, int4, int8, int16, uint, uint2, uint3, uint4, uint8, uint16, long, or ulong.
+SUBGROUP_VEC_TYPES = ['f','i','j']
+for int_type in SUBGROUP_VEC_TYPES:
+	for vecsize in ['2','3','4','8','16']:
+		ret_type = arg_type = 'Dv'+vecsize+'_'+int_type
+		mask_type = 'j'
+		for suffix in ['', '_xor']:
+			generate_function("intel_sub_group_shuffle"+suffix, SIG_TO_LLVM_TYPE_MAP[ret_type], '', False, arg_type, mask_type)
+		for suffix in ['_up', '_down']:
+			generate_function("intel_sub_group_shuffle"+suffix, SIG_TO_LLVM_TYPE_MAP[ret_type], '', False, arg_type, arg_type, mask_type)
+
+for vecsize in ['','2','4','8']:
+	# uints
+	if vecsize:
+		arg_type = 'Dv' + vecsize + '_j'
+	else:
+		arg_type = 'j'
+	ret_type = arg_type
+	PConstArg = 'PKj'
+	PArg = 'Pj'
+	generate_function("intel_sub_group_block_read"+vecsize, SIG_TO_LLVM_TYPE_MAP[ret_type], '', ("global"), PConstArg)
+	generate_function("intel_sub_group_block_write"+vecsize, SIG_TO_LLVM_TYPE_MAP['v'], '', ("global", 'none'), PArg, arg_type)
+	generate_function("intel_sub_group_block_read_ui"+vecsize, SIG_TO_LLVM_TYPE_MAP[ret_type], '', ("global"), PConstArg)
+	generate_function("intel_sub_group_block_write_ui"+vecsize, SIG_TO_LLVM_TYPE_MAP['v'], '', ("global", 'none'), PArg, arg_type)
+	# ushorts
+	if vecsize:
+		arg_type = 'Dv' + vecsize + '_t'
+	else:
+		arg_type = 't'
+	ret_type = arg_type
+	PConstArg = 'PKt'
+	PArg = 'Pt'
+	generate_function("intel_sub_group_block_read_us"+vecsize, SIG_TO_LLVM_TYPE_MAP[ret_type], '', ("global"), PConstArg)
+	generate_function("intel_sub_group_block_write_us"+vecsize, SIG_TO_LLVM_TYPE_MAP['v'], '', ("global", 'none'), PArg, arg_type)
+
 print("""
 
 attributes #0 = { alwaysinline nounwind "correctly-rounded-divide-sqrt-fp-math"="false" "denorms-are-zero"="false" "disable-tail-calls"="false" "less-precise-fpmad"="false" "no-frame-pointer-elim"="true" "no-frame-pointer-elim-non-leaf" "no-infs-fp-math"="false" "no-nans-fp-math"="false" "no-signed-zeros-fp-math"="false" "no-trapping-math"="false" "stack-protector-buffer-size"="8" "unsafe-fp-math"="false" "use-soft-float"="false" }
@@ -1487,11 +1662,10 @@ attributes #0 = { alwaysinline nounwind "correctly-rounded-divide-sqrt-fp-math"=
 !opencl.ocl.version = !{!0}
 !opencl.spir.version = !{!0}
 !llvm.ident = !{!1}
-!llvm.module.flags = !{!2, !3}
+!llvm.module.flags = !{!2}
 
 !0 = !{i32 1, i32 2}
 !1 = !{!"clang version 12.0.0"}
-!2 = !{i32 1, !"wchar_size", i32 4}
-!3 = !{i32 7, !"PIC Level", i32 2}
+!2 = !{i32 7, !"PIC Level", i32 2}
 
 """)
