@@ -60,19 +60,13 @@ using namespace llvm;
 typedef llvm::MemoryLocation Location;
 typedef llvm::AliasResult AliasResult;
 
-#if LLVM_MAJOR < 16
-#define AAResultB AAResultBase<WorkItemAAResult>
-#else
-#define AAResultB AAResultBase
-#endif
-
 /// WorkItemAliasAnalysis - This is a simple alias analysis
 /// implementation that uses pocl metadata to make sure memory accesses from
 /// different work items are not aliasing.
 ///
 
-class WorkItemAAResult : public AAResultB {
-    friend AAResultB;
+class WorkItemAAResult : public AAResultBase {
+    friend AAResultBase;
 
 public:
     static char ID;
@@ -166,31 +160,6 @@ WorkItemAAResult::alias(const Location &LocA, const Location &LocB) {
     return WorkItemAAResult::alias(LocA, LocB);
 }
 
-#if LLVM_MAJOR < MIN_LLVM_NEW_PASSMANAGER
-
-char WorkItemAliasAnalysis::ID = 0;
-
-bool WorkItemAliasAnalysis::runOnFunction(llvm::Function &F) {
-  auto &TLIWP = getAnalysis<TargetLibraryInfoWrapperPass>();
-  auto TLI = TLIWP.getTLI(F);
-  Result.reset(new WorkItemAAResult(TLI));
-  return false;
-}
-
-void WorkItemAliasAnalysis::getAnalysisUsage(AnalysisUsage &AU) const {
-  AU.setPreservesAll();
-  AU.addRequired<TargetLibraryInfoWrapperPass>();
-}
-
-WorkItemAliasAnalysis::WorkItemAliasAnalysis() : FunctionPass(ID), Result() {}
-WorkItemAliasAnalysis::~WorkItemAliasAnalysis() {}
-
-WorkItemAAResult &WorkItemAliasAnalysis::getResult() { return *Result; }
-
-REGISTER_OLD_FANALYSIS(PASS_NAME, PASS_CLASS, PASS_DESC);
-
-#else
-
 llvm::AnalysisKey WorkItemAliasAnalysis::Key;
 
 WorkItemAliasAnalysis::Result
@@ -202,6 +171,4 @@ WorkItemAliasAnalysis::run(llvm::Function &F,
 
 REGISTER_NEW_FANALYSIS(PASS_NAME, PASS_CLASS, PASS_DESC);
 
-#endif
-
 } // namespace pocl
