@@ -33,22 +33,6 @@
 
 namespace pocl {
 
-#if LLVM_MAJOR < MIN_LLVM_NEW_PASSMANAGER
-
-class WorkitemLoops : public llvm::FunctionPass {
-public:
-  static char ID;
-  WorkitemLoops() : FunctionPass(ID) {}
-  virtual ~WorkitemLoops() {}
-  llvm::StringRef getPassName() const override {
-    return "PoCL Work-Item Loops pass";
-  }
-
-  virtual bool runOnFunction(llvm::Function &F) override;
-  virtual void getAnalysisUsage(llvm::AnalysisUsage &AU) const override;
-};
-
-#else
 
 class WorkitemLoops : public llvm::PassInfoMixin<WorkitemLoops> {
 public:
@@ -56,9 +40,14 @@ public:
   llvm::PreservedAnalyses run(llvm::Function &F,
                               llvm::FunctionAnalysisManager &AM);
   static bool isRequired() { return true; }
+
+  // Returns false in case the WG generator can or should not handle the given
+  // kernel. It might refuse to handle trickiest of conditional barrier
+  // scenarios which would result in non-vectorizable loops anyhow.
+  static bool canHandleKernel(llvm::Function &K,
+                              llvm::FunctionAnalysisManager &AM);
 };
 
-#endif
 
 } // namespace pocl
 
