@@ -23,80 +23,18 @@
 */
 
 #include "common.hh"
+#include "pocl_networking.h"
 #include "traffic_monitor.hh"
 
 #include <arpa/inet.h>
 #include <cassert>
-#ifdef ENABLE_VSOCK
+#ifdef HAVE_LINUX_VSOCK_H
 #include <linux/vm_sockets.h>
 #endif
 #include <netdb.h>
 #include <sys/uio.h>
 #include <unistd.h>
 
-ssize_t read_full(int fd, void *p, size_t total, TrafficMonitor *tm) {
-  //      float_time_point start = Time::now();
-
-  size_t readb = 0;
-  ssize_t res;
-  char *ptr = static_cast<char *>(p);
-  if (tm)
-    tm->rxRequested(total);
-  while (readb < total) {
-    size_t remain = total - readb;
-    res = ::read(fd, ptr + readb, remain);
-    if (res < 0) {
-      int e = errno;
-      if (e == EAGAIN || e == EWOULDBLOCK)
-        continue;
-      else
-        return -1;
-    }
-    if (res == 0) { // EOF
-      return 0;
-    }
-    readb += (size_t)res;
-    if (tm)
-      tm->rxConfirmed(res);
-  }
-
-  return static_cast<ssize_t>(total);
-  //      float_time_point end = Time::now();
-  //      float_sec elapsed_seconds = end-start;
-  //      std::cerr << "read " << total << " bytes   @ " <<
-  //      elapsed_seconds.count() << "\n";
-}
-
-int write_full(int fd, void *p, size_t total, TrafficMonitor *tm) {
-  //      float_time_point start = Time::now();
-
-  size_t written = 0;
-  ssize_t res;
-  char *ptr = static_cast<char *>(p);
-  if (tm)
-    tm->txSubmitted(total);
-  while (written < total) {
-    size_t remain = total - written;
-    res = ::write(fd, ptr + written, remain);
-    if (res < 0) {
-      int e = errno;
-      if (e == EAGAIN || e == EWOULDBLOCK)
-        continue;
-      else
-        return -1;
-    }
-    written += (size_t)res;
-    if (tm)
-      tm->txConfirmed(res);
-  }
-  return 0;
-
-  //      float_time_point end = Time::now();
-  //      float_sec elapsed_seconds = end-start;
-  //      std::cerr << "written " << total << " bytes   @ " <<
-  //      elapsed_seconds.count() << "\n";
-}
-
 void replyID(Reply *rep, ReplyMessageType t, uint32_t id) {
   rep->rep.message_type = t;
   rep->rep.failed = 0;
@@ -157,7 +95,7 @@ static const char *const hex_digits = "0123456789abcdef";
 std::string hexdigits(std::string acc, uint8_t x) {
   acc.push_back(hex_digits[(x & 0xf0) >> 4]);
   acc.push_back(hex_digits[(x & 0x0f) >> 0]);
-  return std::move(acc);
+  return acc;
 }
 
 std::string describe_sockaddr(struct sockaddr *addr, unsigned addr_size) {
@@ -168,7 +106,7 @@ std::string describe_sockaddr(struct sockaddr *addr, unsigned addr_size) {
   else if (addr->sa_family == AF_INET6)
     inet_ntop(addr->sa_family, &((struct sockaddr_in6 *)addr)->sin6_addr,
               ip_str.data(), ip_str.capacity());
-#ifdef ENABLE_VSOCK
+#ifdef HAVE_LINUX_VSOCK_H
   else if (addr->sa_family == AF_VSOCK) {
     struct sockaddr_vm *vm_addr = (sockaddr_vm *)addr;
 
@@ -217,6 +155,9 @@ uint64_t transfer_size(const RequestMsg_t &msg) {
   case MessageType_MigrateD2D:
     return msg.m.migrate.size;
     break;
+  case MessageType_Shutdown:
+    return 0;
+    break;
   default:
     assert(!"Unhandled RDMA message");
     return 0;
