@@ -23,6 +23,8 @@
    IN THE SOFTWARE.
 */
 
+#include <memory>
+#include <mutex>
 #include <poll.h>
 #include <sys/socket.h>
 
@@ -36,17 +38,24 @@
 #define POLLRDHUP 0
 #endif
 
-RequestQueueThread::RequestQueueThread(std::atomic_int *f,
+RequestQueueThread::RequestQueueThread(std::shared_ptr<Connection> Conn,
                                        VirtualContextBase *c, ExitHelper *e,
-                                       TrafficMonitor *tm, const char *id_str)
-    : fd(f), virtualContext(c), eh(e), id_str(id_str), netstat(tm) {
-  io_thread = std::thread{&RequestQueueThread::readThread, this};
+                                       const char *id_str)
+    : InboundConnection(Conn), virtualContext(c), eh(e),
+      ThreadIdentifier(id_str) {
+  IOThread = std::thread{&RequestQueueThread::readThread, this};
 }
 
 RequestQueueThread::~RequestQueueThread() {
-  eh->requestExit(id_str.c_str(), 0);
-  io_thread.join();
-  shutdown(*fd, SHUT_RD);
+  eh->requestExit(ThreadIdentifier.c_str(), 0);
+  IOThread.join();
+}
+
+void RequestQueueThread::setConnection(
+    std::shared_ptr<Connection> NewConnection) {
+  std::unique_lock<std::mutex> l(ConnectionGuard);
+  InboundConnection = NewConnection;
+  ConnectionNotifier.notify_one();
 }
 
 void RequestQueueThread::readThread() {
@@ -54,38 +63,60 @@ void RequestQueueThread::readThread() {
 
   struct pollfd pfd;
   pfd.events = POLLIN | POLLRDHUP;
-  int nevs;
+  int NumEvents;
 
-  int fd = *this->fd;
-  int oldfd = fd;
   while (1) {
-    fd = *this->fd;
-    if (fd != oldfd) {
-      POCL_MSG_PRINT_GENERAL("%s: FD change detected: %d -> %d\n",
-                             id_str.c_str(), oldfd, fd);
-    }
-    oldfd = fd;
     if (eh->exit_requested())
       return;
 
-    pfd.fd = fd;
-    nevs = poll(&pfd, 1, 3 * MS_PER_S);
-    if (nevs < 1)
+    std::unique_lock<std::mutex> l(ConnectionGuard);
+
+    if (InboundConnection.get() == nullptr)
+      ConnectionNotifier.wait(l);
+
+    pfd.fd = InboundConnection->pollableFd();
+    /* HACK: Timeout after 1s so peer request threads don't get stuck when the
+     * session is being shut down. */
+    NumEvents = poll(&pfd, 1, 1000);
+    if (NumEvents == 0)
       continue;
-    if (pfd.revents & (POLLERR | POLLNVAL | POLLHUP | POLLRDHUP))
+
+    if (NumEvents < 0) {
+      int e = errno;
+      if (e == EINTR)
+        continue;
+      else {
+        // Either a SERIOUS bug in the poll code above (EFAULT, EINVAL) or the
+        // system is out of memory. Can't really recover from either case at
+        // runtime so let's just bail.
+        eh->requestExit("Fatal error during poll(2) in RequestQueueThread", e);
+        return;
+      }
+    }
+    if (pfd.revents & (POLLERR | POLLNVAL | POLLHUP | POLLRDHUP)) {
+      InboundConnection.reset();
       continue;
+    }
     if (!(pfd.revents & POLLIN))
       continue;
 
-    Request *request = new Request();
-    while (!request->IsFullyRead) {
-      if (!request->read(fd)) {
-        delete request;
+    Request *IncomingRequest = new(std::nothrow) Request();
+    if (IncomingRequest == nullptr) {
+      eh->requestExit("Out of host memory in in RequestQueueThread", ENOMEM);
+      return;
+    }
+    while (!IncomingRequest->IsFullyRead) {
+      if (!IncomingRequest->read(InboundConnection.get())) {
+        delete IncomingRequest;
+        IncomingRequest = nullptr;
+        InboundConnection.reset();
         continue;
       }
     }
 
-    switch (request->req.message_type) {
+    l.unlock();
+
+    switch (IncomingRequest->Body.message_type) {
     case MessageType_ConnectPeer:
     case MessageType_DeviceInfo:
     case MessageType_CreateBuffer:
@@ -109,7 +140,7 @@ void RequestQueueThread::readThread() {
     case MessageType_MigrateD2D:
     case MessageType_RdmaBufferRegistration:
     case MessageType_Shutdown: {
-      virtualContext->nonQueuedPush(request);
+      virtualContext->nonQueuedPush(IncomingRequest);
       break;
     }
     case MessageType_ReadBuffer:
@@ -126,19 +157,19 @@ void RequestQueueThread::readThread() {
     case MessageType_WriteImageRect:
     case MessageType_FillImageRect:
     case MessageType_RunKernel: {
-      virtualContext->queuedPush(request);
+      virtualContext->queuedPush(IncomingRequest);
       break;
     }
     case MessageType_NotifyEvent: {
       // TODO: this message should probably contain an actual status... (see
       // also rdma thread)
-      virtualContext->notifyEvent(request->req.event_id, CL_COMPLETE);
-      delete request;
+      virtualContext->notifyEvent(IncomingRequest->Body.event_id, CL_COMPLETE);
+      delete IncomingRequest;
       break;
     }
 
     default: {
-      virtualContext->unknownRequest(request);
+      virtualContext->unknownRequest(IncomingRequest);
       break;
     }
     }
