@@ -23,10 +23,14 @@
 */
 
 #include <cassert>
+#include <cstdint>
+#include <cstdio>
 #include <memory>
 #include <unordered_set>
 
+#include "CL/cl_ext.h"
 #include "common.hh"
+#include "request.hh"
 
 #ifdef ENABLE_RDMA
 #include "rdma_reply_th.hh"
@@ -56,7 +60,7 @@ SharedContextBase *createSharedCLContext(cl::Platform *platform, size_t pid,
 #define INIT_VARS                                                              \
   int err = CL_SUCCESS;                                                        \
   size_t i, j;                                                                 \
-  uint64_t id = req->req.obj_id;
+  uint64_t id = req->Body.obj_id;
 
 #define FOR_EACH_CONTEXT_DO(CODE)                                              \
   for (i = 0; i < SharedContextList.size(); ++i) {                             \
@@ -90,18 +94,15 @@ public:
 
 class VirtualCLContext : public VirtualContextBase {
   PoclDaemon *Daemon;
-  ReplyQueueThreadUPtr write_slow;
-  ReplyQueueThreadUPtr write_fast;
+  ExitHelper ExitSignal;
+  ReplyQueueThreadUPtr WriteSlow;
+  ReplyQueueThreadUPtr WriteFast;
 #ifdef ENABLE_RDMA
   RdmaReplyThreadUPtr write_rdma;
   RdmaRequestThreadUPtr read_rdma;
 #endif
   PeerHandlerUPtr peers;
   uint32_t peer_id;
-  std::atomic_int command_fd;
-  std::atomic_int stream_fd;
-
-  ExitHelper exit_helper;
 
   std::vector<cl::Platform> PlatformList;
   std::vector<SharedContextBase *> SharedContextList;
@@ -110,9 +111,10 @@ class VirtualCLContext : public VirtualContextBase {
   size_t current_printf_position;
   std::mutex printf_lock;
 
-  std::condition_variable main_cond;
-  std::mutex main_mutex;
-  std::deque<Request *> main_que;
+  std::thread MainThread;
+  std::condition_variable MainCond;
+  std::mutex MainMutex;
+  std::deque<Request *> MainQueue;
 
 #ifdef ENABLE_RDMA
   std::shared_ptr<RdmaConnection> client_rdma;
@@ -126,7 +128,7 @@ class VirtualCLContext : public VirtualContextBase {
   std::mutex client_regions_mutex;
   uint32_t client_uses_rdma;
 #endif
-  TrafficMonitor *netstat;
+  std::shared_ptr<TrafficMonitor> netstat;
 
   std::unordered_set<uint32_t> BufferIDset;
   std::unordered_set<uint32_t> SamplerIDset;
@@ -134,6 +136,7 @@ class VirtualCLContext : public VirtualContextBase {
   std::unordered_set<uint32_t> QueueIDset;
   std::unordered_set<uint32_t> ProgramIDset;
   std::unordered_set<uint32_t> KernelIDset;
+  std::unordered_set<uint32_t> CommandBufferIDSet;
 
   std::unordered_map<uint32_t, ContextVector> ProgramPlatformBuildMap;
 
@@ -143,13 +146,18 @@ public:
 
   ~VirtualCLContext() {
     // stop threads
-    assert(exit_helper.exit_requested());
+    assert(ExitSignal.exit_requested());
+    MainCond.notify_one();
+    if (MainThread.joinable())
+      MainThread.join();
     POCL_MSG_PRINT_GENERAL("VCTX: DEST\n");
 
+    // Wake up IO threads in case they were waiting for a connection
+    WriteFast->setConnection(nullptr);
+    WriteSlow->setConnection(nullptr);
+
     // make sure no shared context tries to broadcast stuff
-    std::unique_lock<std::mutex> lock(main_mutex);
-    // TOFIX: peers cannot be freed without crashing, let them leak for now.
-    // peers.reset();
+    std::unique_lock<std::mutex> Lock(MainMutex);
     for (auto i : SharedContextList) {
       delete i;
     }
@@ -161,8 +169,8 @@ public:
   virtual size_t init(PoclDaemon *d, ClientConnections_t conns,
                       uint64_t session, CreateOrAttachSessionMsg_t &params);
 
-  virtual void updateSockets(std::optional<int> command_fd,
-                             std::optional<int> stream_fd) override;
+  virtual void replaceConnections(std::shared_ptr<Connection> Command,
+                                  std::shared_ptr<Connection> Stream) override;
 
   virtual void nonQueuedPush(Request *req) override;
 
@@ -208,7 +216,7 @@ private:
   void FreeBuffer(Request *req, Reply *rep);
 
   void BuildOrLinkProgram(Request *req, Reply *rep, bool is_binary,
-                          bool is_builtin, bool is_spirv,
+                          bool is_builtin, bool is_dbk, bool is_spirv,
                           bool CompileOnly = false, bool LinkOnly = false);
 
   void FreeProgram(Request *req, Reply *rep);
@@ -217,6 +225,10 @@ private:
 
   void FreeKernel(Request *req, Reply *rep);
 
+  void CreateCommandBuffer(Request *req, Reply *rep);
+
+  void FreeCommandBuffer(Request *req, Reply *rep);
+
   void CreateSampler(Request *req, Reply *rep);
 
   void FreeSampler(Request *req, Reply *rep);
@@ -241,8 +253,6 @@ size_t VirtualCLContext::init(PoclDaemon *d, ClientConnections_t conns,
   Daemon = d;
   current_printf_position = 0;
   TotalDevices = 0;
-  command_fd = conns.fd_command;
-  stream_fd = conns.fd_stream;
   peer_id = params.peer_id;
 #ifdef ENABLE_RDMA
   client_uses_rdma = params.use_rdma;
@@ -252,27 +262,32 @@ size_t VirtualCLContext::init(PoclDaemon *d, ClientConnections_t conns,
 #endif
 
   std::string id_string = std::to_string(session);
-  netstat = new TrafficMonitor(&exit_helper, id_string);
+  netstat.reset(new TrafficMonitor(&ExitSignal, id_string));
+  if (conns.low_latency.get())
+    conns.low_latency->setMeter(netstat);
+  if (conns.bulk_throughput.get())
+    conns.bulk_throughput->setMeter(netstat);
 
 #ifdef ENABLE_RDMA
   if (client_uses_rdma) {
     read_rdma = RdmaRequestThreadUPtr(
-        new RdmaRequestThread(this, &exit_helper, netstat, "RDMA_R", conns.rdma,
+        new RdmaRequestThread(this, &ExitSignal, netstat, "RDMA_R", conns.rdma,
                               &client_mem_regions, &client_regions_mutex));
     write_rdma = RdmaReplyThreadUPtr(
-        new RdmaReplyThread(this, &exit_helper, netstat, "RDMA_W", conns.rdma,
+        new RdmaReplyThread(this, &ExitSignal, netstat, "RDMA_W", conns.rdma,
                             &client_mem_regions, &client_regions_mutex));
   }
 #endif
-  write_slow = ReplyQueueThreadUPtr(
-      new ReplyQueueThread(&stream_fd, this, &exit_helper, netstat, "WT_S"));
-  write_fast = ReplyQueueThreadUPtr(
-      new ReplyQueueThread(&command_fd, this, &exit_helper, netstat, "WT_F"));
+  WriteSlow = ReplyQueueThreadUPtr(
+      new ReplyQueueThread(conns.bulk_throughput, this, &ExitSignal, "WT_S"));
+  WriteFast = ReplyQueueThreadUPtr(
+      new ReplyQueueThread(conns.low_latency, this, &ExitSignal, "WT_F"));
 
   peers = PeerHandlerUPtr(new PeerHandler(peer_id, conns.incoming_peer_mutex,
                                           conns.incoming_peer_queue, this,
-                                          &exit_helper, netstat));
+                                          &ExitSignal, netstat));
   initPlatforms();
+  MainThread = std::move(std::thread(&VirtualCLContext::run, this));
 
   POCL_MSG_PRINT_INFO("Created shared contexts for %" PRIuS
                       " platforms / %" PRIuS " devices\n",
@@ -281,12 +296,17 @@ size_t VirtualCLContext::init(PoclDaemon *d, ClientConnections_t conns,
   return TotalDevices;
 }
 
-void VirtualCLContext::updateSockets(std::optional<int> fd_command,
-                                     std::optional<int> fd_stream) {
-  if (fd_command.has_value())
-    command_fd = fd_command.value();
-  if (fd_stream.has_value())
-    stream_fd = fd_stream.value();
+void VirtualCLContext::replaceConnections(
+    std::shared_ptr<Connection> Latency,
+    std::shared_ptr<Connection> Throughput) {
+  if (Latency.get()) {
+    Latency->setMeter(netstat);
+    WriteFast->setConnection(Latency);
+  }
+  if (Throughput.get()) {
+    Throughput->setMeter(netstat);
+    WriteSlow->setConnection(Throughput);
+  }
 }
 
 size_t VirtualCLContext::initPlatforms() {
@@ -300,7 +320,7 @@ size_t VirtualCLContext::initPlatforms() {
 
   for (size_t i = 0; i < PlatformList.size(); ++i) {
     SharedContextBase *p = createSharedCLContext(
-        &(PlatformList[i]), i, this, write_slow.get(), write_fast.get());
+        &(PlatformList[i]), i, this, WriteSlow.get(), WriteFast.get());
 
     SharedContextList[i] = p;
     TotalDevices += p->numDevices();
@@ -316,16 +336,16 @@ size_t VirtualCLContext::initPlatforms() {
 
 void VirtualCLContext::nonQueuedPush(Request *req) {
 
-  if (req->req.message_type != MessageType_ServerInfo &&
+  if (req->Body.message_type != MessageType_ServerInfo &&
       checkPlatformDeviceValidity(req))
     return;
 
   POCL_MSG_PRINT_GENERAL("VCTX NON-QUEUED PUSH (msg: %" PRIu64 ")\n",
-                         uint64_t(req->req.msg_id));
+                         uint64_t(req->Body.msg_id));
 
-  std::unique_lock<std::mutex> lock(main_mutex);
-  main_que.push_back(req);
-  main_cond.notify_one();
+  std::unique_lock<std::mutex> Lock(MainMutex);
+  MainQueue.push_back(req);
+  MainCond.notify_one();
 }
 
 void VirtualCLContext::queuedPush(Request *req) {
@@ -335,8 +355,8 @@ void VirtualCLContext::queuedPush(Request *req) {
 
   POCL_MSG_PRINT_GENERAL(
       "VCTX QUEUED PUSH (msg: %" PRIu64 ", event: %" PRIu64 ")\n",
-      uint64_t(req->req.msg_id), uint64_t(req->req.event_id));
-  SharedContextList[req->req.pid]->queuedPush(req);
+      uint64_t(req->Body.msg_id), uint64_t(req->Body.event_id));
+  SharedContextList[req->Body.pid]->queuedPush(req);
 }
 
 void VirtualCLContext::notifyEvent(uint64_t event_id, cl_int status) {
@@ -348,35 +368,35 @@ void VirtualCLContext::notifyEvent(uint64_t event_id, cl_int status) {
 }
 
 void VirtualCLContext::requestExit(int code, const char *reason) {
-  exit_helper.requestExit(reason, code);
+  ExitSignal.requestExit(reason, code);
 }
 
 void VirtualCLContext::broadcastToPeers(const Request &req) {
-  std::unique_lock<std::mutex> lock(main_mutex);
+  std::unique_lock<std::mutex> Lock(MainMutex);
   peers->broadcast(req);
 }
 
 void VirtualCLContext::unknownRequest(Request *req) {
   Reply *rep = new Reply(req);
-  POCL_MSG_ERR("Unknown request type: %d\n", req->req.message_type);
-  replyFail(&rep->rep, &req->req, CL_INVALID_OPERATION);
-  write_fast->pushReply(rep);
+  POCL_MSG_ERR("Unknown request type: %d\n", req->Body.message_type);
+  replyFail(&rep->rep, &req->Body, CL_INVALID_OPERATION);
+  WriteFast->pushReply(rep);
 }
 
 int VirtualCLContext::checkPlatformDeviceValidity(Request *req) {
-  if (req->req.message_type == MessageType_RdmaBufferRegistration)
+  if (req->Body.message_type == MessageType_RdmaBufferRegistration)
     return 0;
-  if (req->req.message_type == MessageType_MigrateD2D &&
-      peer_id == req->req.m.migrate.source_peer_id) {
-    uint32_t pid = req->req.m.migrate.source_pid;
-    uint32_t did = req->req.m.migrate.source_pid;
+  if (req->Body.message_type == MessageType_MigrateD2D &&
+      peer_id == req->Body.m.migrate.source_peer_id) {
+    uint32_t pid = req->Body.m.migrate.source_pid;
+    uint32_t did = req->Body.m.migrate.source_pid;
     if ((pid < PlatformList.size()) &&
         (did < SharedContextList[pid]->numDevices()))
       return 0;
   }
 
-  uint32_t pid = req->req.pid;
-  uint32_t did = req->req.did;
+  uint32_t pid = req->Body.pid;
+  uint32_t did = req->Body.did;
   if ((pid < PlatformList.size()) &&
       (did < SharedContextList[pid]->numDevices()))
     return 0;
@@ -385,14 +405,14 @@ int VirtualCLContext::checkPlatformDeviceValidity(Request *req) {
 
   int err =
       (pid < PlatformList.size() ? CL_INVALID_DEVICE : CL_INVALID_PLATFORM);
-  replyFail(&reply->rep, &req->req, err);
+  replyFail(&reply->rep, &req->Body, err);
 
   POCL_MSG_ERR("Message ID %" PRIu64 ": Unknown Platform ID %" PRIu32
                " or Device ID %" PRIu32 "\n",
-               uint64_t(req->req.msg_id), uint32_t(req->req.pid),
-               uint32_t(req->req.did));
+               uint64_t(req->Body.msg_id), uint32_t(req->Body.pid),
+               uint32_t(req->Body.did));
 
-  write_fast->pushReply(reply);
+  WriteFast->pushReply(reply);
   return 1;
 }
 
@@ -403,27 +423,27 @@ int VirtualCLContext::run() {
   Reply *reply;
   while (1) {
 
-    if (exit_helper.exit_requested()) {
-      auto e = exit_helper.status();
+    if (ExitSignal.exit_requested()) {
+      auto e = ExitSignal.status();
       POCL_MSG_PRINT_GENERAL("VCTX: exit req, status: %d\n", e);
       return e;
     }
 
-    std::unique_lock<std::mutex> lock(main_mutex);
-    if (main_que.size() > 0) {
-      Request *request = main_que.front();
-      main_que.pop_front();
-      lock.unlock();
+    std::unique_lock<std::mutex> Lock(MainMutex);
+    if (MainQueue.size() > 0) {
+      Request *request = MainQueue.front();
+      MainQueue.pop_front();
+      Lock.unlock();
 
       reply = nullptr;
-      if (request->req.message_type != MessageType_MigrateD2D &&
-          request->req.message_type != MessageType_RdmaBufferRegistration) {
+      if (request->Body.message_type != MessageType_MigrateD2D &&
+          request->Body.message_type != MessageType_RdmaBufferRegistration) {
         reply = new Reply(request);
       }
 
       // PROCESSS REQUEST, then PUSH REPLY to WRITE Q
 
-      switch (request->req.message_type) {
+      switch (request->Body.message_type) {
       case MessageType_ServerInfo:
         ServerInfo(request, reply);
         break;
@@ -453,27 +473,31 @@ int VirtualCLContext::run() {
         break;
 
       case MessageType_BuildProgramFromBinary:
-        BuildOrLinkProgram(request, reply, true, false, false);
+        BuildOrLinkProgram(request, reply, true, false, false, false);
         break;
 
       case MessageType_BuildProgramFromSource:
-        BuildOrLinkProgram(request, reply, false, false, false);
+        BuildOrLinkProgram(request, reply, false, false, false, false);
         break;
 
       case MessageType_CompileProgramFromSource:
-        BuildOrLinkProgram(request, reply, false, false, false, true);
+        BuildOrLinkProgram(request, reply, false, false, false, false, true);
         break;
 
       case MessageType_BuildProgramFromSPIRV:
-        BuildOrLinkProgram(request, reply, false, false, true);
+        BuildOrLinkProgram(request, reply, false, false, false, true);
         break;
 
       case MessageType_CompileProgramFromSPIRV:
-        BuildOrLinkProgram(request, reply, false, false, true, true);
+        BuildOrLinkProgram(request, reply, false, false, false, true, true);
         break;
 
       case MessageType_BuildProgramWithBuiltins:
-        BuildOrLinkProgram(request, reply, false, true, false);
+        BuildOrLinkProgram(request, reply, false, true, false, false);
+        break;
+
+      case MessageType_BuildProgramWithDefinedBuiltins:
+        BuildOrLinkProgram(request, reply, false, true, true, false);
         break;
 
       case MessageType_LinkProgram:
@@ -492,6 +516,14 @@ int VirtualCLContext::run() {
         FreeKernel(request, reply);
         break;
 
+      case MessageType_CreateCommandBuffer:
+        CreateCommandBuffer(request, reply);
+        break;
+
+      case MessageType_FreeCommandBuffer:
+        FreeCommandBuffer(request, reply);
+        break;
+
       case MessageType_CreateSampler:
         CreateSampler(request, reply);
         break;
@@ -513,7 +545,8 @@ int VirtualCLContext::run() {
         break;
 
       case MessageType_Shutdown:
-        exit_helper.requestExit("Shutdown notification from client", 0);
+        ExitSignal.requestExit("Shutdown notification from client", 0);
+        delete reply;
         return 0;
 
       case MessageType_RdmaBufferRegistration:
@@ -521,8 +554,8 @@ int VirtualCLContext::run() {
         // unused existing fields are being repurposed for rdma info here:
         // uint32_t peer_id, uint32_t buf_id, uint32_t rkey, uint64_t vaddr
         peers->notifyRdmaBufferRegistration(
-            request->req.cq_id, request->req.obj_id, request->req.did,
-            request->req.msg_id);
+            request->Body.cq_id, request->Body.obj_id, request->Body.did,
+            request->Body.msg_id);
 #endif
         // Just ignore, this does not require a reply
         delete request;
@@ -536,11 +569,11 @@ int VirtualCLContext::run() {
         reply->extra_data.clear();
         reply->extra_size = 0;
         POCL_MSG_ERR("Unknown message type received: %" PRIu32 "\n",
-                     uint32_t(request->req.message_type));
+                     uint32_t(request->Body.message_type));
       }
 
       if (reply) {
-        write_fast->pushReply(reply);
+        WriteFast->pushReply(reply);
         // Reply frees the request when destroyed
       }
 
@@ -548,7 +581,7 @@ int VirtualCLContext::run() {
       auto now = std::chrono::system_clock::now();
       std::chrono::duration<unsigned long> d(3);
       now += d;
-      main_cond.wait_until(lock, now);
+      MainCond.wait_until(Lock, now);
     }
   }
 }
@@ -559,10 +592,10 @@ int VirtualCLContext::run() {
 void VirtualCLContext::ConnectPeer(Request *req, Reply *rep) {
   INIT_VARS;
   std::array<uint8_t, AUTHKEY_LENGTH> authkey;
-  std::memcpy(authkey.data(), req->req.m.connect_peer.authkey, AUTHKEY_LENGTH);
-  err = peers->connectPeer(req->req.msg_id, req->req.m.connect_peer.address,
-                           req->req.m.connect_peer.port,
-                           req->req.m.connect_peer.session, authkey);
+  std::memcpy(authkey.data(), req->Body.m.connect_peer.authkey, AUTHKEY_LENGTH);
+  err = peers->connectPeer(req->Body.msg_id, req->Body.m.connect_peer.address,
+                           req->Body.m.connect_peer.port,
+                           req->Body.m.connect_peer.session, authkey);
   RETURN_IF_ERR;
   replyOK(rep, MessageType_ConnectPeerReply);
 }
@@ -573,10 +606,10 @@ void VirtualCLContext::CreateCmdQueue(Request *req, Reply *rep) {
   INIT_VARS;
   CHECK_ID_NOT_EXISTS(QueueIDset, CL_INVALID_COMMAND_QUEUE);
 
-  TP_CREATE_QUEUE(req->req.msg_id, req->req.client_did, id);
-  err = SharedContextList[req->req.pid]->createQueue(
-      id, req->req.did); // TODO queue flags
-  TP_CREATE_QUEUE(req->req.msg_id, req->req.client_did, id);
+  TP_CREATE_QUEUE(req->Body.msg_id, req->Body.client_did, id);
+  err = SharedContextList[req->Body.pid]->createQueue(
+      id, req->Body.did); // TODO queue flags
+  TP_CREATE_QUEUE(req->Body.msg_id, req->Body.client_did, id);
 
   RETURN_IF_ERR;
   QueueIDset.insert(id);
@@ -587,9 +620,9 @@ void VirtualCLContext::FreeCmdQueue(Request *req, Reply *rep) {
   INIT_VARS;
   CHECK_ID_EXISTS(QueueIDset, CL_INVALID_COMMAND_QUEUE);
 
-  TP_FREE_QUEUE(req->req.msg_id, req->req.client_did, id);
-  err = SharedContextList[req->req.pid]->freeQueue(id);
-  TP_FREE_QUEUE(req->req.msg_id, req->req.client_did, id);
+  TP_FREE_QUEUE(req->Body.msg_id, req->Body.client_did, id);
+  err = SharedContextList[req->Body.pid]->freeQueue(id);
+  TP_FREE_QUEUE(req->Body.msg_id, req->Body.client_did, id);
 
   QueueIDset.erase(id);
   RETURN_IF_ERR;
@@ -602,7 +635,7 @@ void VirtualCLContext::CreateBuffer(Request *req, Reply *rep) {
   INIT_VARS;
   CHECK_ID_NOT_EXISTS(BufferIDset, CL_INVALID_MEM_OBJECT);
 
-  CreateBufferMsg_t &m = req->req.m.create_buffer;
+  CreateBufferMsg_t &m = req->Body.m.create_buffer;
 
 #ifdef ENABLE_RDMA
   uint32_t buf_size = m.size;
@@ -613,7 +646,7 @@ void VirtualCLContext::CreateBuffer(Request *req, Reply *rep) {
                                CL_MEM_READ_WRITE, buf_size, 0);
   if (!b) {
     POCL_MSG_ERR("SVM allocation of size %" PRIu32 " failed\n", buf_size);
-    replyFail(&rep->rep, &req->req, CL_MEM_OBJECT_ALLOCATION_FAILURE);
+    replyFail(&rep->rep, &req->Body, CL_MEM_OBJECT_ALLOCATION_FAILURE);
     return;
   }
   std::unique_ptr<char, SVMDeleter> shadow_buf(
@@ -636,7 +669,7 @@ void VirtualCLContext::CreateBuffer(Request *req, Reply *rep) {
     } catch (const std::runtime_error &e) {
       POCL_MSG_ERR("%s", e.what());
       err = 1;
-      replyFail(&rep->rep, &req->req, CL_MEM_OBJECT_ALLOCATION_FAILURE);
+      replyFail(&rep->rep, &req->Body, CL_MEM_OBJECT_ALLOCATION_FAILURE);
       FOR_EACH_CONTEXT_DO(freeBuffer(id, false));
       return;
     }
@@ -648,15 +681,15 @@ void VirtualCLContext::CreateBuffer(Request *req, Reply *rep) {
   rdma_shadow_buffers.insert({id, std::move(shadow_buf)});
 #endif
 
-  TP_CREATE_BUFFER(req->req.msg_id, req->req.client_did, id);
+  TP_CREATE_BUFFER(req->Body.msg_id, req->Body.client_did, id);
 
-  uint64_t devaddr;
-  FOR_EACH_CONTEXT_DO(
-      createBuffer(id, m.size, m.flags, (void *)m.host_ptr, (void **)&devaddr));
+  uint64_t devaddr = 0;
+  FOR_EACH_CONTEXT_DO(createBuffer(id, m.size, m.flags, (void *)m.host_ptr,
+                                   m.parent_id, m.origin, (void **)&devaddr));
   // Do not pass pointer to device_addr directly above since
   // it's a packed struct and the address might be unaligned.
   rep->rep.m.create_buffer.device_addr = devaddr;
-  TP_CREATE_BUFFER(req->req.msg_id, req->req.client_did, id);
+  TP_CREATE_BUFFER(req->Body.msg_id, req->Body.client_did, id);
   FOR_EACH_CONTEXT_UNDO(freeBuffer(id, false));
 
   RETURN_IF_ERR;
@@ -676,12 +709,12 @@ void VirtualCLContext::CreateBuffer(Request *req, Reply *rep) {
 
 void VirtualCLContext::FreeBuffer(Request *req, Reply *rep) {
   INIT_VARS;
-  if (!req->req.m.free_buffer.is_svm)
+  if (!req->Body.m.free_buffer.is_svm)
     CHECK_ID_EXISTS(BufferIDset, CL_INVALID_MEM_OBJECT);
 
-  TP_FREE_BUFFER(req->req.msg_id, req->req.client_did, id);
-  FOR_EACH_CONTEXT_DO(freeBuffer(id, req->req.m.free_buffer.is_svm));
-  TP_FREE_BUFFER(req->req.msg_id, req->req.client_did, id);
+  TP_FREE_BUFFER(req->Body.msg_id, req->Body.client_did, id);
+  FOR_EACH_CONTEXT_DO(freeBuffer(id, req->Body.m.free_buffer.is_svm));
+  TP_FREE_BUFFER(req->Body.msg_id, req->Body.client_did, id);
 
   BufferIDset.erase(id);
 #ifdef ENABLE_RDMA
@@ -706,12 +739,12 @@ void VirtualCLContext::FreeBuffer(Request *req, Reply *rep) {
 
 void VirtualCLContext::BuildOrLinkProgram(Request *req, Reply *rep,
                                           bool is_binary, bool is_builtin,
-                                          bool is_spirv, bool CompileOnly,
-                                          bool LinkOnly) {
+                                          bool is_dbk, bool is_spirv,
+                                          bool CompileOnly, bool LinkOnly) {
   INIT_VARS;
   CHECK_ID_NOT_EXISTS(ProgramIDset, CL_INVALID_PROGRAM);
 
-  BuildProgramMsg_t &m = req->req.m.build_program;
+  BuildProgramMsg_t &m = req->Body.m.build_program;
 
   POCL_MSG_PRINT_GENERAL(
       "VirtualCTX: %s %s program %" PRIu64 " for %" PRIu32 " devices\n",
@@ -720,7 +753,7 @@ void VirtualCLContext::BuildOrLinkProgram(Request *req, Reply *rep,
                  : (is_builtin ? "builtin" : (is_spirv ? "SPIR-V" : "source"))),
       id, uint32_t(m.num_devices));
   // source / binary / builtin / SPIR-V must be provided as a payload
-  assert(req->extra_size > 0);
+  assert(req->ExtraDataSize > 0);
   assert(m.num_devices);
   std::vector<uint32_t> DevList{};
   ContextVector ProgramContexts;
@@ -729,27 +762,27 @@ void VirtualCLContext::BuildOrLinkProgram(Request *req, Reply *rep,
   std::unordered_map<uint64_t, std::vector<unsigned char>> InputBinaries;
   std::unordered_map<uint64_t, std::string> build_logs;
   size_t num_kernels = 0;
-  char *source = (char *)(req->extra_data.data());
+  char *source = (char *)(req->ExtraData.data());
   size_t source_len = m.payload_size;
-  char *options = (char *)(req->extra_data2.data());
+  char *options = (char *)(req->ExtraData2.data());
 
   if (LinkOnly) {
     // We receive N client cl_program IDs to link. The first
     // 32b int is the number.
     assert(m.payload_size > sizeof(uint32_t));
-    uint32_t NumPrograms = *((uint32_t *)req->extra_data.data());
+    uint32_t NumPrograms = *((uint32_t *)req->ExtraData.data());
     assert(m.payload_size == sizeof(uint32_t) + sizeof(uint32_t) * NumPrograms);
 
     // Just add the program ids as keys.
     for (size_t i = 0; i < NumPrograms; ++i) {
-      uint32_t ClientProgramID = ((uint32_t *)req->extra_data.data())[i + 1];
+      uint32_t ClientProgramID = ((uint32_t *)req->ExtraData.data())[i + 1];
       InputBinaries[ClientProgramID] = std::vector<unsigned char>();
     }
   } else if (is_binary || is_spirv) {
     source_len = 0;
     unsigned char *buffer = (unsigned char *)source;
     source = nullptr;
-    assert(req->extra_size == m.payload_size);
+    assert(req->ExtraDataSize == m.payload_size);
     unsigned char *buf = buffer;
     size_t buffer_size = m.payload_size;
     uint32_t n_binaries;
@@ -764,13 +797,13 @@ void VirtualCLContext::BuildOrLinkProgram(Request *req, Reply *rep,
 
       std::vector<unsigned char> binary(buf, buf + bin_size);
       buf += bin_size;
-      assert((buf - buffer) <= (size_t)m.payload_size);
+      assert((size_t)(buf - buffer) <= (size_t)m.payload_size);
       uint64_t id = ((uint64_t)m.platforms[i] << 32) + m.devices[i];
       InputBinaries[id] = std::move(binary);
     }
   }
 
-  TP_BUILD_PROGRAM(req->req.msg_id, req->req.client_did, id);
+  TP_BUILD_PROGRAM(req->Body.msg_id, req->Body.client_did, id);
   for (i = 0; i < SharedContextList.size(); ++i) {
     DevList.clear();
     for (j = 0; j < m.num_devices; ++j) {
@@ -779,16 +812,16 @@ void VirtualCLContext::BuildOrLinkProgram(Request *req, Reply *rep,
     }
     if (DevList.size() > 0) {
       err = SharedContextList[i]->buildOrLinkProgram(
-          id, DevList, source, source_len, is_binary, is_builtin, is_spirv,
-          options, InputBinaries, OutputBinaries, build_logs, num_kernels,
-          m.svm_region_offset, CompileOnly, LinkOnly);
+          id, DevList, source, source_len, is_binary, is_builtin, is_dbk,
+          is_spirv, options, InputBinaries, OutputBinaries, build_logs,
+          num_kernels, m.svm_region_offset, CompileOnly, LinkOnly);
       if (err == CL_SUCCESS) {
         ProgramContexts.push_back(SharedContextList[i]);
       } else
         break;
     }
   }
-  TP_BUILD_PROGRAM(req->req.msg_id, req->req.client_did, id);
+  TP_BUILD_PROGRAM(req->Body.msg_id, req->Body.client_did, id);
 
   // output reply
   rep->extra_data.resize(MAX_REMOTE_BUILDPROGRAM_SIZE);
@@ -859,11 +892,11 @@ void VirtualCLContext::FreeProgram(Request *req, Reply *rep) {
 
   ContextVector &contexts = ProgramPlatformBuildMap[id];
 
-  TP_FREE_PROGRAM(req->req.msg_id, req->req.client_did, id);
+  TP_FREE_PROGRAM(req->Body.msg_id, req->Body.client_did, id);
   for (i = 0; i < contexts.size(); ++i) {
     err = contexts[i]->freeProgram(id);
   }
-  TP_FREE_PROGRAM(req->req.msg_id, req->req.client_did, id);
+  TP_FREE_PROGRAM(req->Body.msg_id, req->Body.client_did, id);
 
   ProgramIDset.erase(id);
   RETURN_IF_ERR;
@@ -874,20 +907,20 @@ void VirtualCLContext::CreateKernel(Request *req, Reply *rep) {
   INIT_VARS;
   CHECK_ID_NOT_EXISTS(KernelIDset, CL_INVALID_KERNEL);
 
-  CreateKernelMsg_t &m = req->req.m.create_kernel;
+  CreateKernelMsg_t &m = req->Body.m.create_kernel;
   CHECK_ID_EXISTS2(ProgramPlatformBuildMap, CL_INVALID_PROGRAM,
                    uint32_t(m.prog_id));
 
   ContextVector &contexts = ProgramPlatformBuildMap[m.prog_id];
 
-  TP_CREATE_KERNEL(req->req.msg_id, req->req.client_did, id);
+  TP_CREATE_KERNEL(req->Body.msg_id, req->Body.client_did, id);
   for (i = 0; i < contexts.size(); ++i) {
     err = contexts[i]->createKernel(id, m.prog_id,
-                                    (const char *)(req->extra_data.data()));
+                                    (const char *)(req->ExtraData.data()));
     if (err != CL_SUCCESS)
       break;
   }
-  TP_CREATE_KERNEL(req->req.msg_id, req->req.client_did, id);
+  TP_CREATE_KERNEL(req->Body.msg_id, req->Body.client_did, id);
   if (err != CL_SUCCESS) {
     for (j = 0; j < i; ++j) {
       err = contexts[i]->freeKernel(id);
@@ -903,36 +936,91 @@ void VirtualCLContext::FreeKernel(Request *req, Reply *rep) {
   INIT_VARS;
   CHECK_ID_EXISTS(KernelIDset, CL_INVALID_KERNEL);
 
-  FreeKernelMsg_t &m = req->req.m.free_kernel;
+  FreeKernelMsg_t &m = req->Body.m.free_kernel;
   CHECK_ID_EXISTS2(ProgramPlatformBuildMap, CL_INVALID_PROGRAM,
                    uint32_t(m.prog_id));
 
   ContextVector &contexts = ProgramPlatformBuildMap[m.prog_id];
 
-  TP_FREE_KERNEL(req->req.msg_id, req->req.client_did, req->req.obj_id);
+  TP_FREE_KERNEL(req->Body.msg_id, req->Body.client_did, req->Body.obj_id);
   for (i = 0; i < contexts.size(); ++i) {
     err = contexts[i]->freeKernel(id);
   }
-  TP_FREE_KERNEL(req->req.msg_id, req->req.client_did, req->req.obj_id);
+  TP_FREE_KERNEL(req->Body.msg_id, req->Body.client_did, req->Body.obj_id);
 
   KernelIDset.erase(id);
   RETURN_IF_ERR;
   replyOK(rep, MessageType_FreeKernelReply);
 }
 
-/****************************************************************************************************************/
+/******************************************************************************/
+
+void VirtualCLContext::CreateCommandBuffer(Request *req, Reply *rep) {
+  INIT_VARS;
+  CHECK_ID_NOT_EXISTS(CommandBufferIDSet, CL_INVALID_COMMAND_BUFFER_KHR);
+
+  CreateCommandBufferMsg_t &m = req->Body.m.create_cmdbuf;
+
+  std::vector<Request *> Commands;
+  std::vector<uint32_t> Devices;
+  std::vector<uint32_t> Queues;
+
+  Queues.reserve(m.num_queues);
+  for (size_t i = 0; i < m.num_queues; ++i) {
+    uint32_t QueueIndex;
+    ::memcpy(&QueueIndex,
+             req->ExtraData.data() + m.queues_offset + i * sizeof(uint32_t),
+             sizeof(uint32_t));
+    Queues.push_back(QueueIndex);
+  }
+
+  Devices.push_back(req->Body.did);
+
+  Commands.reserve(m.num_commands);
+  ByteReader Reader(req->ExtraData.data() + m.commands_offset, m.commands_size);
+  while (!Reader.eof()) {
+    Request *R = new Request{};
+    if (R->readFull(&Reader))
+      Commands.push_back(R);
+  }
+
+  assert(Commands.size() == m.num_commands);
+  TP_CREATE_COMMAND_BUFFER(req->Body.msg_id, req->Body.client_did, id);
+  err = SharedContextList[req->Body.pid]->createCommandBuffer(id, Devices,
+                                                              Queues, Commands);
+  TP_CREATE_COMMAND_BUFFER(req->Body.msg_id, req->Body.client_did, id);
+
+  RETURN_IF_ERR;
+  CommandBufferIDSet.insert(id);
+  replyID(rep, MessageType_CreateCommandBufferReply, id);
+}
+
+void VirtualCLContext::FreeCommandBuffer(Request *req, Reply *rep) {
+  INIT_VARS;
+  CHECK_ID_EXISTS(CommandBufferIDSet, CL_INVALID_COMMAND_BUFFER_KHR);
+
+  TP_FREE_COMMAND_BUFFER(req->Body.msg_id, req->Body.client_did, id);
+  err = SharedContextList[req->Body.pid]->freeCommandBuffer(id);
+  TP_FREE_COMMAND_BUFFER(req->Body.msg_id, req->Body.client_did, id);
+
+  RETURN_IF_ERR;
+  CommandBufferIDSet.erase(id);
+  replyOK(rep, MessageType_FreeCommandBufferReply);
+}
+
+/******************************************************************************/
 
 void VirtualCLContext::CreateImage(Request *req, Reply *rep) {
   INIT_VARS;
   CHECK_ID_NOT_EXISTS(ImageIDset, CL_INVALID_MEM_OBJECT);
 
-  CreateImageMsg_t &m = req->req.m.create_image;
+  CreateImageMsg_t &m = req->Body.m.create_image;
 
-  TP_CREATE_IMAGE(req->req.msg_id, req->req.client_did, id);
+  TP_CREATE_IMAGE(req->Body.msg_id, req->Body.client_did, id);
   FOR_EACH_CONTEXT_DO(createImage(
       id, m.flags, m.channel_order, m.channel_data_type, m.type, m.width,
       m.height, m.depth, m.array_size, m.row_pitch, m.slice_pitch));
-  TP_CREATE_IMAGE(req->req.msg_id, req->req.client_did, id);
+  TP_CREATE_IMAGE(req->Body.msg_id, req->Body.client_did, id);
 
   FOR_EACH_CONTEXT_UNDO(freeImage(id));
 
@@ -945,9 +1033,9 @@ void VirtualCLContext::FreeImage(Request *req, Reply *rep) {
   INIT_VARS;
   CHECK_ID_EXISTS(ImageIDset, CL_INVALID_MEM_OBJECT);
 
-  TP_FREE_IMAGE(req->req.msg_id, req->req.client_did, req->req.obj_id);
+  TP_FREE_IMAGE(req->Body.msg_id, req->Body.client_did, req->Body.obj_id);
   FOR_EACH_CONTEXT_DO(freeImage(id));
-  TP_FREE_IMAGE(req->req.msg_id, req->req.client_did, req->req.obj_id);
+  TP_FREE_IMAGE(req->Body.msg_id, req->Body.client_did, req->Body.obj_id);
 
   ImageIDset.erase(id);
   RETURN_IF_ERR;
@@ -960,12 +1048,12 @@ void VirtualCLContext::CreateSampler(Request *req, Reply *rep) {
   INIT_VARS;
   CHECK_ID_NOT_EXISTS(SamplerIDset, CL_INVALID_SAMPLER);
 
-  CreateSamplerMsg_t &m = req->req.m.create_sampler;
+  CreateSamplerMsg_t &m = req->Body.m.create_sampler;
 
-  TP_CREATE_SAMPLER(req->req.msg_id, req->req.client_did, id);
+  TP_CREATE_SAMPLER(req->Body.msg_id, req->Body.client_did, id);
   FOR_EACH_CONTEXT_DO(
       createSampler(id, m.normalized, m.address_mode, m.filter_mode));
-  TP_CREATE_SAMPLER(req->req.msg_id, req->req.client_did, id);
+  TP_CREATE_SAMPLER(req->Body.msg_id, req->Body.client_did, id);
 
   FOR_EACH_CONTEXT_UNDO(freeSampler(id));
 
@@ -978,9 +1066,9 @@ void VirtualCLContext::FreeSampler(Request *req, Reply *rep) {
   INIT_VARS;
   CHECK_ID_EXISTS(SamplerIDset, CL_INVALID_SAMPLER);
 
-  TP_FREE_SAMPLER(req->req.msg_id, req->req.client_did, req->req.obj_id);
+  TP_FREE_SAMPLER(req->Body.msg_id, req->Body.client_did, req->Body.obj_id);
   FOR_EACH_CONTEXT_DO(freeSampler(id));
-  TP_FREE_SAMPLER(req->req.msg_id, req->req.client_did, req->req.obj_id);
+  TP_FREE_SAMPLER(req->Body.msg_id, req->Body.client_did, req->Body.obj_id);
 
   SamplerIDset.erase(id);
   RETURN_IF_ERR;
@@ -990,8 +1078,8 @@ void VirtualCLContext::FreeSampler(Request *req, Reply *rep) {
 /****************************************************************************************************************/
 
 void VirtualCLContext::MigrateD2D(Request *req) {
-  MigrateD2DMsg_t &m = req->req.m.migrate;
-  RequestMsg_t &r = req->req;
+  MigrateD2DMsg_t &m = req->Body.m.migrate;
+  RequestMsg_t &r = req->Body;
   EventTiming_t evt{};
   uint32_t mem_obj_id = r.obj_id;
   uint32_t size_buffer_id = m.size_id;
@@ -1023,7 +1111,7 @@ void VirtualCLContext::MigrateD2D(Request *req) {
           (m.is_image == 0 ? "Buffer" : "Image"), uint32_t(m.source_pid),
           uint32_t(m.source_did), fake_ev_id, uint64_t(r.msg_id));
 #ifdef ENABLE_RDMA
-      req->extra_data.clear();
+      req->ExtraData.clear();
 #ifndef RDMA_USE_SVM
       // No SVM, we have actual shadow buffers. Write data to shadow buffer but
       // do not pass it along as extra_data, rdma thread will fetch the
@@ -1032,8 +1120,8 @@ void VirtualCLContext::MigrateD2D(Request *req) {
 #endif
 #else
       // No RDMA, no persistent shadow buffers
-      req->extra_data.resize(m.size);
-      storage = (char *)(req->extra_data.data());
+      req->ExtraData.resize(m.size);
+      storage = (char *)(req->ExtraData.data());
 #endif
 
 #ifndef RDMA_USE_SVM
@@ -1066,7 +1154,7 @@ void VirtualCLContext::MigrateD2D(Request *req) {
 
     /* Write extra_size after possible content size has been read, just before
      * pushing the request on */
-    req->extra_size = m.size;
+    req->ExtraDataSize = m.size;
 
     // .... and now we can push the writeBuffer to the queue
     if (m.dest_peer_id == peer_id) {
@@ -1104,7 +1192,8 @@ void VirtualCLContext::DeviceInfo(Request *req, Reply *rep) {
   // The first string starts at offset 1.
   rep->rep.strings_size = 1;
 
-  SharedContextList[req->req.pid]->getDeviceInfo(req->req.did, info, strings);
+  SharedContextList[req->Body.pid]->getDeviceInfo(req->Body.did, info, strings,
+                                                  req->Body.m.device_info.id);
 
   for (const std::string &str : strings)
     rep->rep.strings_size += str.size() + 1;
@@ -1136,6 +1225,3 @@ VirtualContextBase *createVirtualContext(PoclDaemon *d,
   return vctx;
 }
 
-void startVirtualContextMainloop(VirtualContextBase *ctx) {
-  static_cast<VirtualCLContext *>(ctx)->run();
-}
