@@ -27,8 +27,7 @@
 
 // Enable OpenCL C++ exceptions
 #define CL_HPP_ENABLE_EXCEPTIONS
-#define CL_HPP_MINIMUM_OPENCL_VERSION 120
-#define CL_HPP_TARGET_OPENCL_VERSION 120
+
 #include <CL/opencl.hpp>
 
 #include <cstdio>
@@ -63,7 +62,7 @@ main(void)
     // Pick first platform
     cl_context_properties cprops[] = {
       CL_CONTEXT_PLATFORM, (cl_context_properties)(platformList[0])(), 0};
-    cl::Context context(CL_DEVICE_TYPE_CPU|CL_DEVICE_TYPE_GPU, cprops);
+    cl::Context context(CL_DEVICE_TYPE_ALL, cprops);
 
     // Query the set of devices attched to the context
     std::vector<cl::Device> devices = context.getInfo<CL_CONTEXT_DEVICES>();
