@@ -3,8 +3,7 @@
 #include "pocl_opencl.h"
 
 #define CL_HPP_ENABLE_EXCEPTIONS
-#define CL_HPP_MINIMUM_OPENCL_VERSION 120
-#define CL_HPP_TARGET_OPENCL_VERSION 120
+
 #define CL_HPP_CL_1_2_DEFAULT_BUILD
 #include <CL/opencl.hpp>
 #include <iostream>
@@ -52,13 +51,12 @@ __kernel void pocltest(int xarg1, int xarg2) {
 }
 )RAW";
 
-int main(int argc, char *argv[])
-{
-    cl::Platform platform = cl::Platform::getDefault();
-    cl::Device device = cl::Device::getDefault();
-    try {
-        cl::CommandQueue queue = cl::CommandQueue::getDefault();
-        cl::Program program(SOURCE, true);
+int main(int, char **) {
+  cl::Platform platform = cl::Platform::getDefault();
+  cl::Device device = cl::Device::getDefault();
+  try {
+    cl::CommandQueue queue = cl::CommandQueue::getDefault();
+    cl::Program program(SOURCE, true);
 
 #if (__GNUC__ > 5)
 #pragma GCC diagnostic push
