@@ -9,8 +9,7 @@
 #include "pocl_opencl.h"
 
 #define CL_HPP_ENABLE_EXCEPTIONS
-#define CL_HPP_MINIMUM_OPENCL_VERSION 120
-#define CL_HPP_TARGET_OPENCL_VERSION 120
+
 #define CL_HPP_CL_1_2_DEFAULT_BUILD
 #include <CL/opencl.hpp>
 
@@ -30,20 +29,20 @@ __kernel void __attribute__ ((reqd_work_group_size(128, 1, 1))) grudge_assign_0(
 }
 )RAW";
 
-int main(int argc, char *argv[]) {
+int main(int, char **) {
   int n = 8;
-  unsigned successful = 0;
+  int successful = 0;
 
   cl::Platform platform = cl::Platform::getDefault();
   cl::Device device = cl::Device::getDefault();
   try {
     cl::CommandQueue queue = cl::CommandQueue::getDefault();
-    cl::Program program(SOURCE, true);
 
     if (poclu_supports_extension(device.get(), "cl_khr_fp64") == 0) {
       std::cout << "this test requires cl_khr_fp64, test SKIPPED\n";
       return 77;
     }
+    cl::Program program(SOURCE, true);
 
     // Create buffers on the device.
     cl::Buffer buffer_A(CL_MEM_READ_WRITE, sizeof(double) * n);
