@@ -12,11 +12,12 @@ main(void)
   cl_image_format *img_formats;
   cl_uint num_entries;
 
+  cl_platform_id pid = NULL;
   cl_context ctx = NULL;
   cl_device_id dev = NULL;
   cl_command_queue cq = NULL;
-  
-  err = poclu_get_any_device(&ctx, &dev, &cq);
+
+  err = poclu_get_any_device2 (&ctx, &dev, &cq, &pid);
   CHECK_OPENCL_ERROR_IN("poclu_get_any_device");
 
   TEST_ASSERT(dev != NULL);
@@ -47,7 +48,7 @@ main(void)
 
   free (img_formats);
 
-  CHECK_CL_ERROR (clUnloadCompiler ());
+  CHECK_CL_ERROR (clUnloadPlatformCompiler (pid));
 
   printf ("OK\n");
   return EXIT_SUCCESS;
